/*
 * Copyright 1998-2005 VIA Technologies, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * VIA, AND/OR ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef VIA_ACE_H
#define VIA_ACE_H
#include "padlock.h"
#include <malloc.h>
#include <memory.h>
#ifdef __cplusplus
extern "C" {
#endif

// The Maximum Size for padlock_ace_alloc_buf is 64kb 
#define MAX_CIPHER_BUFFER_SIZE            (64 * 1024)

#define NEH_GEN_KEY 	0x00000000	// generate key schedule
#define NEH_LOAD_KEY 	0x00000080	// load key schedule from memory

#define NEH_ENCRYPT 	0x00000000	// encryption
#define NEH_DECRYPT 	0x00000200	// decryption

#define NEH_KEY128 	0x000 + 0x00a	// 128 bit key
#define NEH_KEY192 	0x400 + 0x00c	// 192 bit key
#define NEH_KEY256 	0x800 + 0x00e	// 256 bit key


extern int asm_ace_available();
extern int asm_ace2_available();
extern void asm_ace_ecb_op5(void *key, void *ctrl, void *src, void *dst, int count);
extern void asm_ace_cbc_op6(void *key, void *ctrl, void *src, void *dst, int count, void *iv);
extern void asm_ace_ofb_op6(void *key, void *ctrl, void *src, void *dst, int count, void *iv);
extern void asm_ace_cfb_op6(void *key, void *ctrl, void *src, void *dst, int count, void *iv);
extern void asm_ace_ctr_op6(void *key, void *ctrl, void *src, void *dst, int count, void *iv);


struct ace_aes_context
{
	ACE_AES_MODE mode;
	KEY_LENGTH key_length;
	unsigned char *iv;
	unsigned char last_iv[16];
	unsigned int encrypt_extended_key[60];
	unsigned int decrypt_extended_key[60];
};

struct aligned_memory_context
{
	int size;
	int offset;                     // the length of buf used
	unsigned char *temp_buf;
	unsigned char *p_alignedbuf;
};
#ifdef Linux
	#ifndef uint_64
		typedef unsigned long long uint_64;
	#endif
#else
	#include <windows.h>
	#ifndef uint_64
		typedef UINT64 uint_64;
	#endif
#endif

#ifdef _X64
#define ALIGN16(x) (unsigned char *)((((uint_64)x) + 15 )&0xfffffffffffffff0)
#else
#define ALIGN16(x) (unsigned char *)((((unsigned int)x) + 15 )&0xfffffff0)
#endif

#ifdef _X64
#define UNALIGNED(addr) ((uint_64)(addr) & 0x000000000000000F)
#else
#define UNALIGNED(addr) ((unsigned int)(addr) & 0x0000000F)
#endif




#ifndef Linux
int ace_ctr(
		const void *key, 
		const void *ctrl, 
		const void *src, 
		void *dst, 
		int count,
		void *iv);
#endif


AES_RESULT
ace_aes_atomic_crypt(struct ace_aes_context *ctx, 
                          int enc,
                          unsigned char *src, 
						  unsigned char *dst, 
						  int nbytes);


		
#if defined(__cplusplus)
}
#endif

#endif /* VIA_ACE_H */
